#include <FS.h>                   //system plików SPIFFS
#include <ESP8266WiFi.h>          

#include <DNSServer.h>
#include <ESP8266WebServer.h>
#include <WiFiManager.h>          //https://github.com/tzapu/WiFiManager
#include <ArduinoJson.h>          //https://github.com/bblanchon/ArduinoJson

//wyjścia
#define PM25 4
#define PM10 5
#define LED 13

char smog_TX_server[40];
String response,tnij;
WiFiClient client;

//wejścia
#define TRIGGER_PIN 14
#define CALIBRATE_PIN 12

int i;
int delta=1;
int PM25_i=0;
int PM10_i=0;
int PM25_disp=0;
int PM10_disp=0;
int retry=0;

//flag for saving data
bool shouldSaveConfig = false;

//callback notifying us of the need to save config
void saveConfigCallback () {
  Serial.println("Should save config");
  shouldSaveConfig = true;
}


void setup() {
  Serial.begin(115200);
  Serial.println("\nSMOG RX");
  pinMode(TRIGGER_PIN, INPUT);
  pinMode(CALIBRATE_PIN, INPUT);
  pinMode(LED, OUTPUT);
  digitalWrite(LED,LOW);
  pinMode(PM25, OUTPUT);
  digitalWrite(PM25,LOW);
  pinMode(PM10, OUTPUT);
  digitalWrite(PM10,LOW); 
  //clean FS, for testing
  // SPIFFS.format();

  //read configuration from FS json
  Serial.println("mounting FS...");

  if (SPIFFS.begin()) {
    Serial.println("mounted file system");
    if (SPIFFS.exists("/config.json")) {
      //file exists, reading and loading
      Serial.println("reading config file");
      File configFile = SPIFFS.open("/config.json", "r");
      if (configFile) {
        Serial.println("opened config file");
        size_t size = configFile.size();
        // Allocate a buffer to store contents of the file.
        std::unique_ptr<char[]> buf(new char[size]);
        configFile.readBytes(buf.get(), size);
        DynamicJsonBuffer jsonBuffer;
        JsonObject& json = jsonBuffer.parseObject(buf.get());
        json.printTo(Serial);
        if (json.success()) {
          Serial.println("\nparsed json");
          strcpy(smog_TX_server, json["smog_TX_server"]);
        }
        else {
          Serial.println("failed to load json config");
        }
      }
    }
  }
  else {
    Serial.println("failed to mount FS");
  }
  //end read
  // The extra parameters to be configured (can be either global or just in the setup)
  // After connecting, parameter.getValue() will get you the configured value
  // id/name placeholder/prompt default length
  WiFiManagerParameter custom_smog_TX_server("smog_TX_server", "SMOG TX server", smog_TX_server, 40);
  //WiFiManager
  //Local intialization. Once its business is done, there is no need to keep it around
  WiFiManager wifiManager;

  //set config save notify callback
  wifiManager.setSaveConfigCallback(saveConfigCallback);
  
  //add all your parameters here
  wifiManager.addParameter(&custom_smog_TX_server);

  //reset settings - for testing
  //wifiManager.resetSettings();

//----------------------------------------------------------------- tryb ustawiania WiFi
  
if ( digitalRead(TRIGGER_PIN) == LOW ) {
  Serial.println("\n WIFI CONFIG MODE");
  if (!wifiManager.startConfigPortal("SMOG_RX_SETUP")) {
    Serial.println("failed to connect and hit timeout");
  }
  Serial.print("Connected! Device IP is: ");
  Serial.print(WiFi.localIP());
  Serial.println("");
  strcpy(smog_TX_server, custom_smog_TX_server.getValue());
  //save the custom parameters to FS
  if (shouldSaveConfig) {
    Serial.println("saving config");
    DynamicJsonBuffer jsonBuffer;
    JsonObject& json = jsonBuffer.createObject();
    json["smog_TX_server"] = smog_TX_server;
    File configFile = SPIFFS.open("/config.json", "w");
    if (!configFile) {
      Serial.println("failed to open config file for writing");
    }
    json.printTo(Serial);
    json.printTo(configFile);
    configFile.close();
    //end save
  }
}
else{
//----------------------------------------------------------------- tryb kalibracji wskaźników
  if(digitalRead(CALIBRATE_PIN) == LOW ){
    Serial.println("\nMETER CALIBRATE MODE");
    analogWrite(PM25, 1023);
    analogWrite(PM10, 1023);
    while(1) //pętla niekończona
      delay(100); //ale opóźnienie żeby system miał dla siebie czas, inaczej reset watchdoga
  }
//----------------------------------------------------------------- normalny tryb
  else{
    Serial.println("\nREGULAR RUN MODE");
    for (i=0;i<1023;i++){ //machnięcie wskaźnikami do oporu
      analogWrite(PM25, i);
      analogWrite(PM10, i);
      delay(2);
    }
    for (i=1023;i>512;i--){ // i z powrotem
      analogWrite(PM25, i);
      analogWrite(PM10, i);
      delay(2);
    }
    while (WiFi.status() != WL_CONNECTED) { //podczas łączenia z siecią machanie około środka
      analogWrite(PM25, i);
      analogWrite(PM10, i);
      i+=delta;
      if(i>=600)
        delta=-1;
      if(i<=400)
        delta=1;
      delay(2);
    }
    //połączony 
    Serial.print("\nConnected!\nDevice IP is: ");
    Serial.println(WiFi.localIP());
    Serial.print("\nSMOG TX IP is: ");
    Serial.print(smog_TX_server);
    Serial.print("\n");
    analogWrite(PM25, 0);
    analogWrite(PM10, 0);
  }
 }
}

void loop() {
  while (!client.connect(smog_TX_server, 50000)) {
    Serial.print("C");
    if(!(retry%2))
      digitalWrite(LED,HIGH);
    else
      digitalWrite(LED,LOW);
    delay(500);
    retry++;
  }
  digitalWrite(LED,LOW);
  Serial.println("Connected to server");  
  client.print(String("SMOG?"));
  delay(10);
  while (!client.available());
  while (client.available()){
    response = client.readStringUntil('\r');
    Serial.println(response);
    if (response.indexOf("25") == 2) {
      Serial.println("znalazlem 25");
      tnij=response.substring(5);
      Serial.println(tnij);
      PM25_i=tnij.toInt();
      Serial.print("INT PM25=");
      Serial.println(PM25_i);
      if(PM25_i>120)
        PM25_i=120;
      PM25_disp=map(PM25_i,0,120,0,1023);
      Serial.print("INT PM25_disp=");
      Serial.println(PM25_disp);
      analogWrite(PM25,PM25_disp);
    } 
    if (response.indexOf("10") == 2) {
      Serial.println("znalazlem 10");
      tnij=response.substring(5);
      Serial.println(tnij);
      PM10_i=tnij.toInt();
      Serial.print("INT PM10=");
      Serial.println(PM10_i);
      if(PM10_i>200)
        PM10_i=200;
      PM10_disp=map(PM10_i,0,200,0,1023);
      Serial.print("INT PM10_disp=");
      Serial.println(PM10_disp);
      analogWrite(PM10,PM10_disp);
    } 
  }
  //client.flush();
  //client.stop();
  digitalWrite(LED,HIGH);
  delay(100);
  digitalWrite(LED,LOW);
  delay(1000);
}
